#!/usr/bin/env node


'use strict';

var _marked = [prompt].map(regeneratorRuntime.mark);

var yargs = require('yargs');
var chalk = require('chalk');

var PluginManager = require('./plugin.js');
var pe;
var showLogo = require('./logo.js');
var pkg = require('../package.json');
var co = require('co');
var log = require('./log.js');
var inquirer = require('inquirer');
var fullname = require('fullname');
var _ = require('lodash');
var autocomplete = require('inquirer-autocomplete-prompt');

inquirer.registerPrompt('autocomplete', autocomplete);

var BACK = 'alismbiboback';

function buildChoices(choices) {
    return _.flattenDeep([new inquirer.Separator('== 插件列表 =='), choices, new inquirer.Separator('== 功能区=='), {
        name: '返回',
        value: BACK
    }]);
}

var argv = yargs.options({
    v: {
        alias: 'version',
        demand: false,
        describe: chalk.gray('output the version number')
    }
}).usage(chalk.bold('\nUsage:') + ' $0 ' + chalk.blue('[plugin]')).help('h').alias('h', 'help').describe('help', chalk.gray('show help infomation')).locale('en').argv;

if (argv.version || argv.v) {
    showLogo(pkg.version);
}

pe = new PluginManager(argv.yeoman);

co(regeneratorRuntime.mark(function _callee() {
    return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
            switch (_context.prev = _context.next) {
                case 0:
                    if (!(argv._.length > 0)) {
                        _context.next = 5;
                        break;
                    }

                    _context.next = 3;
                    return pe.run(argv._[0]);

                case 3:
                    _context.next = 6;
                    break;

                case 5:
                    return _context.delegateYield(prompt(), 't0', 6);

                case 6:
                case 'end':
                    return _context.stop();
            }
        }
    }, _callee, this);
}));

function prompt() {
    var name, installedPlugins, ret, answers, runAnswers, choices, installAnswer, updateAnswer;
    return regeneratorRuntime.wrap(function prompt$(_context2) {
        while (1) {
            switch (_context2.prev = _context2.next) {
                case 0:
                    _context2.next = 2;
                    return fullname();

                case 2:
                    name = _context2.sent;
                    installedPlugins = pe.getAllInstalled();
                    _context2.prev = 4;
                    _context2.next = 7;
                    return inquirer.prompt([{
                        type: 'list',
                        name: 'type',
                        message: 'Hello! ' + (name || '') + ' \u9009\u62E9\u60F3\u6267\u884C\u7684\u64CD\u4F5C\uFF1A',
                        choices: _.flatten([{
                            name: '运行脚手架',
                            value: 'run'
                        }, {
                            name: '安装脚手架',
                            value: 'install'
                        }, {
                            name: '更新脚手架',
                            value: 'update'
                        }, {
                            name: '退出',
                            value: 'exit'
                        }])
                    }]);

                case 7:
                    answers = _context2.sent;
                    _context2.next = 13;
                    break;

                case 10:
                    _context2.prev = 10;
                    _context2.t0 = _context2['catch'](4);

                    console.log(_context2.t0);

                case 13:
                    if (!(answers.type == 'run')) {
                        _context2.next = 28;
                        break;
                    }

                    if (!installedPlugins.length) {
                        _context2.next = 26;
                        break;
                    }

                    _context2.next = 17;
                    return inquirer.prompt([{
                        type: 'list',
                        name: 'plugin',
                        message: '选择要运行的插件：',
                        choices: buildChoices(installedPlugins.map(function (pl) {
                            return {
                                name: pe.prefix + '-' + pl,
                                value: pl
                            };
                        }))
                    }]);

                case 17:
                    runAnswers = _context2.sent;

                    if (!(runAnswers.plugin == BACK)) {
                        _context2.next = 23;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't1', 20);

                case 20:
                    return _context2.abrupt('return', _context2.t1);

                case 23:
                    return _context2.abrupt('return', pe.run(runAnswers.plugin));

                case 24:
                    _context2.next = 28;
                    break;

                case 26:
                    log.info('没有可用的插件');
                    process.exit(0);

                case 28:
                    if (!(answers.type == 'exit')) {
                        _context2.next = 30;
                        break;
                    }

                    return _context2.abrupt('return');

                case 30:
                    if (!(answers.type == 'install')) {
                        _context2.next = 59;
                        break;
                    }

                    _context2.next = 33;
                    return pe.getAllNpm();

                case 33:
                    choices = _context2.sent;

                    if (!(choices.length >= 10)) {
                        _context2.next = 40;
                        break;
                    }

                    _context2.next = 37;
                    return inquirer.prompt([{
                        type: 'autocomplete',
                        message: '输入要安装的插件',
                        name: 'plugin',
                        source: function source(answers, input) {

                            return new Promise(function (resolve, reject) {
                                setTimeout(function () {
                                    resolve(choices.filter(function (v) {
                                        return new RegExp('^' + input, 'i').exec(v.name) != null;
                                    }));
                                }, 50);
                            });
                        }
                    }]);

                case 37:
                    installAnswer = _context2.sent;
                    _context2.next = 49;
                    break;

                case 40:
                    if (!(choices.length > 0)) {
                        _context2.next = 47;
                        break;
                    }

                    choices = choices.map(function (v) {
                        return {
                            name: pe.prefix + '-' + v.name + ' -- ' + chalk.gray(v.desc),
                            value: v.name
                        };
                    });
                    _context2.next = 44;
                    return inquirer.prompt([{
                        type: 'list',
                        message: '选择要安装的插件',
                        name: 'plugin',
                        choices: buildChoices(choices)
                    }]);

                case 44:
                    installAnswer = _context2.sent;
                    _context2.next = 49;
                    break;

                case 47:
                    log.info('没有可用的插件');
                    process.exit(0);

                case 49:
                    if (!(installAnswer.plugin == BACK)) {
                        _context2.next = 53;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't2', 51);

                case 51:
                    _context2.next = 59;
                    break;

                case 53:
                    _context2.next = 55;
                    return pe.install(installAnswer.plugin);

                case 55:
                    ret = _context2.sent;

                    if (!ret) {
                        _context2.next = 59;
                        break;
                    }

                    log.info('\u4F60\u53EF\u4EE5\u76F4\u63A5\u4F7F\u7528bibo-' + installAnswer.plugin + '\u8FD0\u884C\u811A\u624B\u67B6');
                    return _context2.delegateYield(prompt(), 't3', 59);

                case 59:
                    if (!(answers.type == 'update')) {
                        _context2.next = 77;
                        break;
                    }

                    if (!installedPlugins.length) {
                        _context2.next = 75;
                        break;
                    }

                    _context2.next = 63;
                    return inquirer.prompt([{
                        type: 'list',
                        message: '选择要更新的插件',
                        name: 'plugin',
                        choices: buildChoices(installedPlugins.map(function (v) {
                            return {
                                name: pe.prefix + '-' + v,
                                value: v
                            };
                        }))
                    }]);

                case 63:
                    updateAnswer = _context2.sent;

                    if (!(updateAnswer.plugin == BACK)) {
                        _context2.next = 68;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't4', 66);

                case 66:
                    _context2.next = 73;
                    break;

                case 68:
                    _context2.next = 70;
                    return pe.update(updateAnswer.plugin);

                case 70:
                    ret = _context2.sent;

                    if (!ret) {
                        _context2.next = 73;
                        break;
                    }

                    return _context2.delegateYield(prompt(), 't5', 73);

                case 73:
                    _context2.next = 77;
                    break;

                case 75:

                    log.info('没有可用的插件');
                    process.exit(0);

                case 77:
                case 'end':
                    return _context2.stop();
            }
        }
    }, _marked[0], this, [[4, 10]]);
};